@@ -26,6 +26,8 @@
 #include "dh-assistant-view.h"
 #include "dh-link.h"
 #include "dh-util.h"
+#include "dh-book-manager.h"
+#include "dh-book.h"
 #include "dh-window.h"
 
 typedef struct {
@@ -161,9 +163,18 @@ find_in_buffer (const gchar *buffer,
         return NULL;
 }
 
-static void
-assistant_view_set_link (DhAssistantView *view,
-                         DhLink          *link)
+/**
+ * dh_assistant_view_set_link:
+ * @view: an devhelp assistant view
+ * @link: the #DhLink
+ *
+ * Open @link in the assistant view, if %NULL the view will be blanked.
+ *
+ * Return value: %TRUE if the requested link is open, %FALSE otherwise.
+ **/
+gboolean
+dh_assistant_view_set_link (DhAssistantView *view,
+                            DhLink          *link)
 {
         DhAssistantViewPriv *priv;
         gchar               *uri;
@@ -178,12 +189,12 @@ assistant_view_set_link (DhAssistantView *view,
         const gchar         *start;
         const gchar         *end;
 
-        g_return_if_fail (DH_IS_ASSISTANT_VIEW (view));
+        g_return_val_if_fail (DH_IS_ASSISTANT_VIEW (view), FALSE);
 
         priv = GET_PRIVATE (view);
 
         if (priv->link == link) {
-                return;
+                return TRUE;
         }
 
         if (priv->link) {
@@ -194,8 +205,8 @@ assistant_view_set_link (DhAssistantView *view,
         if (link) {
                 link = dh_link_ref (link);
         } else {
-                webkit_web_view_open (WEBKIT_WEB_VIEW (view), "about:blank");
-                return;
+                webkit_web_view_load_uri (WEBKIT_WEB_VIEW (view), "about:blank");
+                return TRUE;
         }
 
         uri = dh_link_get_uri (link);
@@ -206,16 +217,16 @@ assistant_view_set_link (DhAssistantView *view,
                 g_free (uri);
         } else {
                 g_free (uri);
-                return;
+                return FALSE;
         }
 
         if (g_str_has_prefix (filename, "file://"))
             offset = 7;
-        
+
         file = g_mapped_file_new (filename + offset, FALSE, NULL);
         if (!file) {
                 g_free (filename);
-                return;
+                return FALSE;
         }
 
         contents = g_mapped_file_get_contents (file);
@@ -297,7 +308,7 @@ assistant_view_set_link (DhAssistantView *view,
                                                           "assistant",
                                                           "assistant.js",
                                                           NULL);
-                
+
                 html = g_strdup_printf (
                         "<html>"
                         "<head>"
@@ -334,7 +345,7 @@ assistant_view_set_link (DhAssistantView *view,
 
                 g_free (html);
         } else {
-                webkit_web_view_open (WEBKIT_WEB_VIEW (view), "about:blank");
+                webkit_web_view_load_uri (WEBKIT_WEB_VIEW (view), "about:blank");
         }
 
 #if GLIB_CHECK_VERSION(2,21,3)
@@ -344,6 +355,8 @@ assistant_view_set_link (DhAssistantView *view,
 #endif
 
         g_free (filename);
+
+        return TRUE;
 }
 
 gboolean
@@ -351,11 +364,12 @@ dh_assistant_view_search (DhAssistantView *view,
                           const gchar     *str)
 {
         DhAssistantViewPriv *priv;
-        GList               *keywords, *l;
         const gchar         *name;
         DhLink              *link;
         DhLink              *exact_link;
         DhLink              *prefix_link;
+        DhBookManager       *book_manager;
+        GList               *books;
 
         g_return_val_if_fail (DH_IS_ASSISTANT_VIEW (view), FALSE);
         g_return_val_if_fail (str, FALSE);
@@ -373,45 +387,54 @@ dh_assistant_view_search (DhAssistantView *view,
         g_free (priv->current_search);
         priv->current_search = g_strdup (str);
 
-        keywords = dh_base_get_keywords (dh_assistant_view_get_base (view));
+        book_manager = dh_base_get_book_manager (dh_assistant_view_get_base (view));
 
         prefix_link = NULL;
         exact_link = NULL;
-        for (l = keywords; l && exact_link == NULL; l = l->next) {
-                DhLinkType type;
 
-                link = l->data;
+        for (books = dh_book_manager_get_books (book_manager);
+             !exact_link && books;
+             books = g_list_next (books)) {
+                GList *l;
 
-                type = dh_link_get_link_type (link);
+                for (l = dh_book_get_keywords (DH_BOOK (books->data));
+                     l && exact_link == NULL;
+                     l = l->next) {
+                        DhLinkType type;
 
-                if (type == DH_LINK_TYPE_BOOK ||
-                    type == DH_LINK_TYPE_PAGE ||
-                    type == DH_LINK_TYPE_KEYWORD) {
-                        continue;
-                }
+                        link = l->data;
 
-                name = dh_link_get_name (link);
-                if (strcmp (name, str) == 0) {
-                        exact_link = link;
-                }
-                else if (g_str_has_prefix (name, str)) {
-                        /* Prefer shorter prefix matches. */
-                        if (!prefix_link) {
-                                prefix_link = link;
+                        type = dh_link_get_link_type (link);
+
+                        if (type == DH_LINK_TYPE_BOOK ||
+                            type == DH_LINK_TYPE_PAGE ||
+                            type == DH_LINK_TYPE_KEYWORD) {
+                                continue;
                         }
-                        else if (strlen (dh_link_get_name (prefix_link)) > strlen (name)) {
-                                prefix_link = link;
+
+                        name = dh_link_get_name (link);
+                        if (strcmp (name, str) == 0) {
+                                exact_link = link;
+                        }
+                        else if (g_str_has_prefix (name, str)) {
+                                /* Prefer shorter prefix matches. */
+                                if (!prefix_link) {
+                                        prefix_link = link;
+                                }
+                                else if (strlen (dh_link_get_name (prefix_link)) > strlen (name)) {
+                                        prefix_link = link;
+                                }
                         }
                 }
         }
 
         if (exact_link) {
                 /*g_print ("exact hit: '%s' '%s'\n", exact_link->name, str);*/
-                assistant_view_set_link (view, exact_link);
+                dh_assistant_view_set_link (view, exact_link);
         }
         else if (prefix_link) {
                 /*g_print ("prefix hit: '%s' '%s'\n", prefix_link->name, str);*/
-                assistant_view_set_link (view, prefix_link);
+                dh_assistant_view_set_link (view, prefix_link);
         } else {
                 /*g_print ("no hit\n");*/
                 /*assistant_view_set_link (view, NULL);*/
